import numpy as np
import aesara
import os

backend = os.environ.get('PBNTK_BACKEND')
if backend == 'AESARA':
   AESARA = True
   print('Selecting AESARA backend')
else:
   AESARA = False
   print('Selecting THEANO backend')

class Object:
    pass
cfg = Object()
cfg.lyrs=[None]
cfg.nlayer=0
data = Object()


if AESARA:
   from aesara.sandbox.rng_mrg import MRG_RandomStream as RandomStreams
   from aesara.tensor.random.utils import RandomStream as RandomStreams2
else:
   from aesara.sandbox.rng_mrg import MRG_RandomStreams as RandomStreams
   from aesara.tensor.shared_randomstreams import RandomStreams as RandomStreams2

MAXLYR = 10
def set_defaults():
    # Set defaults
    #cfg.mod_type = 0
    cfg.fast_s2 = 0
    cfg.lbl_type = 0
    cfg.UseCrossEntropy = 1
    cfg.csp_is_running = False
    cfg.csp_fn = None
    cfg.knn_k = 5
    cfg.use_knn = 0
    cfg.invert_NL = 0
    cfg.last_batch = 0
    cfg.dataset_type = 0
    cfg.save_gmm = 0
    cfg.save_while_train = 0
    cfg.windowlen = 1
    cfg.window_shift = 1
    cfg.gmm_use_h = False
    cfg.use_sp_pred5 = False
    cfg.vae_beta = 0
    cfg.lpo_scale = 1
    cfg.aec_std = 0
    cfg.do_class = 1
    cfg.ban_fft  = False 
    cfg.fe_cls_fn = None
    cfg.dbnpbnsyn = 0
    cfg.iclass_wt=0
    cfg.dpbn_ilyr = 0
    cfg.aadata = 0
    cfg.dbn_gibbs_fn = None
    cfg.fe_class_fn = None
    cfg.use_vae = 0
    cfg.fwd_s2 =  1
    #cfg.train_class = 'mn389_0'
    cfg.train_class = 'mn389_0'
    cfg.test_class = 'mn389_0'
    cfg.prefix   = 'tut1'
    #cfg.prefix   = 'testnl'
    #cfg.train_class = 'bfmcsp'
    #cfg.test_class = 'bfmcsp'
    #cfg.prefix   = 'bfmcsp'
    cfg.n_syn = 10
    # [FG]
    cfg.aa_n = 10
    cfg.aa_eps = 1
    cfg.aa_target = -1
    cfg.iplot_pbn = 0
    cfg.dbn_nk = 1
    cfg.rbm_rnd = 0
    cfg.dbn_rnd = 0
    cfg.rbm_trainwts = 1
    cfg.minvar=.0001
    cfg.syn_sort_lik = False
    cfg.syn_use_gmm =0
    cfg.use_maxpool =0
    cfg.partition_train=1
    cfg.sp_hist=3
    cfg.drv_train=False
    cfg.histlen=10000
    cfg.ihist = cfg.histlen
    cfg.lhist=None
    cfg.vhistlen=1000
    cfg.vihist = cfg.vhistlen
    cfg.vlhist=None
    cfg.hist_axs = None
    cfg.hist_fig = None
    cfg.iepoch = 0
    cfg.gmm_mod = 1
    cfg.gmm_lyr = 0
    cfg.gmm_std = 0.001
    cfg.Z_out = None
    cfg.lab_out = None
    cfg.iplot=0
    cfg.LABVAL= 5 # label signal (prior to nonlin) will be [-LABVAL to LABVAL]
    cfg.LABSTD = .242 # results in avg log-lik=0
    cfg.use_scan=True
    if aesara.config.floatX == "float64":
       cfg.amin = 1e-12
       cfg.e_thr = 1e-10
    elif aesara.config.floatX == "float32":
       cfg.amin = 1e-6
       cfg.e_thr = 1e-5
    cfg.assume_s1 = True  # Gaussian layer with assumed variance 1
    cfg.update_s2_in = 0  # update s2 in pbn
    cfg.update_s2 = 0  # update s2 in pbn
    cfg.s2_out = 1.0
    cfg.es_last_col = False
    cfg.do_hump = False
    cfg.untied    = 0
    cfg.C = 0 # classifier separation 
    cfg.use_recon_bias=0
    cfg.use_recon_wts=0
    cfg.conv_ignore_border = True
    cfg.fpbn_use_short = True
    cfg.fpbn_crossent = 0
    cfg.norm_deriv=True
    cfg.fpbn_use_dropout = 0
    cfg.spe_is_running=False
    cfg.dnn_is_running=False
    cfg.init       = 0
    cfg.fpbn_fast = False
    cfg.lr         = 1e-5
    cfg.decay      = 1.00
    cfg.batchsize  = 25
    cfg.epochs     = 50
    cfg.seed       = 15
    cfg.isrand     = 0 # affects input data
    cfg.i_syn_rnd = 0 # affects synthesis
    cfg.aec_syn_rnd = 0 # affects synthesis
    cfg.method     =  'adam'
    cfg.ilayer     = 0
    cfg.inshft = (0,0)
    cfg.cshft = (0,0)
    cfg.do_profile = False
    cfg.aec_is_running = False 
    cfg.gpbn_is_running = False 
    cfg.aec_train_fn = None
    cfg.vae_train_fn = None
    cfg.spe_fn=None
    cfg.use_sigmoid = 0
    cfg.verbosity = 0
    cfg.isave = 0
    cfg.nit_sp_ng = 9
    cfg.updn_is_running = False
    cfg.momentum = 0.8 # momentum
    cfg.l2_reg = 1e-8
    cfg.dnn_fn  = None
    cfg.drv_type = 1
    cfg.fpbn_train_fn = None
    cfg.fpbn_eval_fn  = None
    cfg.fpbn_drv_fn  = None
    cfg.fpbn_train_fn_short = None
    cfg.gpbn_train_fn = None
    cfg.fpbn_is_running = False
    cfg.last_lp=0
    cfg.last_c = 0
    cfg.fpbn_pfx=''
    cfg.fpbn_idat= -1 
    cfg.fwd_fn = None
    cfg.sp_fn = [None]*MAXLYR
    cfg.lyrs = [None]*MAXLYR
    cfg.sp_final_fn = [None]*MAXLYR
    cfg.fetch_train_fn = None 
    cfg.fetch_train_lab_fn = None
    cfg.eval_data=1 # 0=train, 1=valid, 2=test
    cfg.updn_train_fn = None
    cfg.train_data  = None
    cfg.rbm_lyr  = 0
    cfg.show_lyr  = 0
    cfg.valid_data  = None
    cfg.test_data  = None
    cfg.train_labs  = None
    cfg.valid_labs  = None
    cfg.test_labs  = None
    cfg.input_pp = None
    cfg.input_yb = None
    cfg.dbn_eval_fn = [None]*MAXLYR
    cfg.dbn_train_fn = [None]*MAXLYR
    cfg.cdbn_eval_fn = [None]*MAXLYR
    cfg.cdbn_train_fn = [None]*MAXLYR
    cfg.rbm_input_data = [None]*MAXLYR
    cfg.nonlin_in = 0


def inits( ):
    print('Initializing random generator with seed ', cfg.seed)
    np.random.seed(cfg.seed)
    cfg.numpy_rng = np.random.RandomState(cfg.seed)
    cfg.aesara_rng = RandomStreams(cfg.numpy_rng.randint(2**30))
    cfg.aesara_rng2 = RandomStreams2(cfg.numpy_rng.randint(2**30))

