% creates data set for PBN Toolkit  from features created by CSF Toolkit
% 
% transpose_data=1 if feature files created by python
% function make_binaries(outclass,data_classes,feat_type,fmt,ntrain,nvalid,transpose_data,nonlin)
% any data left over after ntrain,nvalid go to test (ntest)
% fmt='default' --> [outclass].mat 
% fmt='stream' --> (binaries + [outclass]_all.txt
%     [outclass]_all.txt is copied to [outclass]_train.txt 
%     ntrain/nvalid are ignored for 'stream' mode. 
% note: 'default' requires identical data shape in all
%    events and does not handle j-function
%
% if ntrain/nvalid are arrays of event numbers, these are used to select the data

function make_binaries(outclass,data_classes,feat_type,fmt,ntrain,nvalid,transpose_data,nonlin,oneclass_in)
    data_directory='./data'; 
    nev_tot=0;
    M=length(data_classes);
    one_hot=M;
    nclass_out = M;
    global oneclass;
    oneclass=oneclass_in; % if 1, assumes just one class, and one label type

    % get parameters of this feature type
    modelfile = [feat_type '.mod'];
    lyrs=read_model_file(modelfile);
    if ~strcmp(lyrs{1}.type,'hann3') || ~strcmp(lyrs{1}.parms{1}.name,'N'),
        fprintf('Error: Model type %s does not start with hann3, or param N not there\n',feat_type);
        return
    end;
    NFFT=lyrs{1}.parms{1}.val;

    if strcmp(fmt,'stream'),
       feat_ddir = [data_directory filesep 'features' filesep  outclass  ];
       [sc,msg] = mkdir(feat_ddir);
       %ev_fn =  [feat_ddir filesep  'event_list.txt'];
       ev_fn =  ['./' outclass  '_all.txt'];
       fpe=fopen(ev_fn,'w');
    else,
       X=[]; tch=[]; jout=[]; icnt=0;
    end
    for i=1:M,
        mj=0;
        data=scan_data(data_directory,data_classes{i});
        c_w_d = pwd;
        for j=1:data.n_events,
            d.n_events=1;
            d.events{1}=data.events{j};
            [Z,Idx,J]=read_event_feat(d, data_directory, data_classes{i},  ...
                     feat_type,data_classes{i},NFFT,transpose_data);
            mj=mj+mean(J{1});
            [pt,nm,ex] = fileparts(d.events{1}.fn);
            [dim,nt]=size(Z{1});
            if strcmp(fmt,'stream'),
               if(oneclass),
                  tcht=Idx{1};
               else,
                  tcht=zeros(M,nt); tcht(i,:)=1;
               end;
               feat_fn = [feat_ddir filesep  data_classes{i} '_' nm '.mat'];
               fn1 = [c_w_d filesep feat_fn];
               fprintf(fpe,'%s\n',fn1);
               write_matlab_stream_file(feat_fn,Z{1},J{1},tcht,nonlin,data_classes);
            else,
               tcht=zeros(M,1); tcht(i)=1;
               icnt=icnt+1;
               [N2,N1]=size(Z{1});
               X{icnt}=Z{1}(:);
               jout{icnt}=sum(J{1}(:));
               tch{icnt}=tcht;
            end;
        end;
        fprintf('Mean J per segment for class %s is %f\n',data_classes{i},mj/data.n_events);
    end; 
    if strcmp(fmt,'stream'),
       fclose(fpe);
       fprintf('Wrote event list to %s\n',ev_fn);
       trfn=  ['./' outclass  '_train.txt'];
       [s,t]=unix(['cp ', ev_fn ,  ' ', trfn]);
       fprintf('Copied to  %s\n',trfn);
    else,
      tch=[tch{:}];
      X=[X{:}];
      jout=[jout{:}];
      [dim,ntot]=size(X);
      [rs,is]=sort(rand(ntot,1));
      ntest=ntot-ntrain-nvalid;
      X2=X(:,is(1:ntest)); tch2=tch(:,is(1:ntest));
      Xv=X(:,is(ntest+1:ntest+nvalid)); tchv=tch(:,is(ntest+1:ntest+nvalid));
      X=X(:,is(ntest+nvalid+1:end)); tch=tch(:,is(ntest+nvalid+1:end));

      j2=jout(is(1:ntest)); 
      jv=jout(is(ntest+1:ntest+nvalid)); 
      j=jout(is(ntest+nvalid+1:end)); 
      fn = ['./',outclass,'.mat'];
      nclass=M; nchan=1; 
      fprintf('Saving to %s, N1=%d, N2=%d, nchan=%d, nonlin=%d,nclass=%d\n', ...
            fn,N1,N2,nchan,nonlin,nclass);
      whos  X Xv X2 tch tchv tch2 j jv j2
      save(fn, 'X','tch','Xv','tchv','X2','tch2','N1','N2','nclass','nchan','nonlin','j','jv','j2');
    end;
    return
    % concatenate by class
    Iclass=[Iclass{:}];
    Z=[Z{:}];

    % randomize event order
    rand('seed',16);
    [rs,is]=sort(rand(1,nev_tot));
    Iclass=Iclass(is);
    Z=Z(is);

    % pull out test data
    Zt=Z(1:ntest);
    Z=Z(ntest+1:end);
    Iclasst=Iclass(1:ntest);
    Iclass=Iclass(ntest+1:end);

    
    % sort Test data according to class
    nt=length(Iclasst);
    ct=zeros(nt,1);
    for i=1:nt, ct(i)=median(Iclasst{i}); end; 
    [cs,is]=sort(ct);
    Iclasst=Iclasst(is);
    Zt=Zt(is);

    % write test data  
    feat_ddir = [data_directory filesep 'features' filesep  outclass  ];
    [st,msg,id] = mkdir(feat_ddir);
    bin_fn = [feat_ddir filesep feat_type '_tst.bin'];
    write_binary(bin_fn,Zt,Iclasst,nclass_out,one_hot);
    
    % write train data
    feat_ddir = [data_directory filesep 'features' filesep  outclass ];
    [st,msg,id] = mkdir(feat_ddir);
    bin_fn = [feat_ddir filesep feat_type '_trn.bin'];
    write_binary(bin_fn,Z,Iclass,nclass_out,one_hot)
    
     
return

 

function [Z,Idx,J]=read_event_feat(data, data_directory, dataclass, feat_type,class_names,NFFT,transpose_data)
    global oneclass;
    if(oneclass),
       M=1;
    else,
       M=length(class_names);
    end;
    dc = dataclass;
    %fprintf('collect feat for data class %s, model class %s\n',  dc, feat_type);
    nev = length(data.events);
    feat_ddir = [data_directory filesep 'features' filesep  dataclass  ];
    data_ddir = [data_directory filesep 'events' filesep  dataclass  ];
    [st,msg]=mkdir(feat_ddir);
    Idx=cell(1,nev);
    Z=cell(1,nev);
    J=cell(1,nev);
    icnt=0;
    for iev=1:nev,   
        t = data.events{iev}.t;
        %fprintf('Getting event %d\n',iev);
        fn = data.events{iev}.fn;
        [pt,nm,ex]=fileparts(fn);
        feat_fn = [feat_ddir filesep feat_type '_' nm '.mat'];
        if ~exist(feat_fn,'file'),
            fprintf('skipping non-existent file: %s\n',feat_fn);
            z=[];
        else,
            load(feat_fn,'z','j'); 
            if(any(isinf(z(:))) || any(isnan(z(:)))),
               fprintf('error, inf or nan in %s\n', feat_fn)
               z=[];
               return
            end;
            if transpose_data, z=z'; end;
            icnt=icnt+1;
            
            % look for label file
            if(M>0),
                mat_fn = [pt filesep  nm '.mat'];
                lab_fn = [pt filesep  nm '.lab'];
                if(exist(mat_fn,'file')),
                    idx=load(mat_fn,'Idx');
                    idx=idx.Idx{1};
                    lab_fn=mat_fn;
                elseif(exist(lab_fn,'file')),
                    idx=read_label_file(lab_fn);
                else,
                    idx={};
                end;
                [d,nseg]=size(z);
                tch=zeros(M,nseg);
                if ~isempty(idx), 
                    fs=data.events{iev}.fs;
                    % these are the starting and ending times of the data in the segments
                    dt = (NFFT/3/fs);
                    t0s=[0:nseg-1]*dt;
                    t1s=[1:nseg]*dt;
                    fprintf('Found label file %s with %d labels\n',lab_fn,size(idx,1));
                    for i=1:size(idx,1),
                        icls=find(strcmp(class_names,idx{i,2}));
                        if isempty(icls),
                              if(oneclass),
                                fprintf('Warning - using label %s\n',idx{i,2});
                                icls=1;
                              end;
                        end;
                        t=idx{i,1};
                        
                        % those segments that are fully inside the range
                        ii=find(t0s>=t(1) & t1s<t(2));
                        tch(icls,ii)=1;
                        
                        % those segments that fully enclose the range
                        ii=find(t0s<=t(1) & t1s>=t(2));
                        if ~isempty(ii),
                            frac= (t(2)-t(1))/dt;
                            tch(icls,ii)=frac;
                        end;
                        
                        % those segments that start in the range, but end outside
                        ii=find(t0s>=t(1) & t0s<=t(2) & t1s >= t(2));
                        if ~isempty(ii),
                            frac= (t(2)-t0s(ii) )/dt;
                            tch(icls,ii)=frac;
                        end;
                        
                        % those segments that start before the range, but
                        % end inside
                        ii=find(t0s<=t(1) & t1s>=t(1) & t1s <= t(2));
                        if ~isempty(ii),
                            frac= (t1s(ii)-t(1))/dt;
                            tch(icls,ii)=frac;
                        end;
                    end;
                    %figure(2)
                    %imagesc(tch)
                    %colormap hot
                    drawnow
                end;
                Idx{icnt}=tch;
            end; %if(M>0),
        end;
        %fprintf('   Feat size=%d by %d\n',size(z,1),size(z,2));
        Z{icnt}=z;
        J{icnt}=j;
    end;
    Z=Z(1:icnt);
    J=J(1:icnt);
    Idx=Idx(1:icnt);
    %sum([Idx{:}],2)
    %pause
return

function data=scan_data(data_directory,dataclass)
    data=[];
    fprintf('Preparing to scan %s\n', dataclass);
    list_file = [data_directory filesep 'events' filesep dataclass filesep 'event_list.txt'];
    if ~exist(list_file,'file'),
        fprintf('event list %s not found\n',list_file);
        return
    end;
    fp=fopen(list_file,'r');
    nev=0;
    while(1),
        if feof(fp), break; end;
        txt=fgetl(fp);
        if isempty(txt) || txt(1)=='#', continue;   end;
        [fn,chnr,fs,t1,t2]=strread(txt,'%s %f %f %f %f');
        nev=nev+1;
    end;
    fclose(fp);
    fprintf('found %d events \n',nev);
    data.n_events=nev;
    fp=fopen(list_file,'r');
    data.events=cell(1,nev);
    nev=0;
    while(1),
        if feof(fp), break; end;
        txt=fgetl(fp);
        if isempty(txt) || txt(1)=='#', continue;   end;
        [fn,chnr,fs,t1,t2]=strread(txt,'%s %d %f %f %f');
        %fprintf('%s\n',txt);
        nev=nev+1;
        data.events{nev}.fn=fn{1};
        data.events{nev}.chnr=chnr;
        data.events{nev}.fs=fs;
        data.events{nev}.t=[t1 t2];
    end;
    fclose(fp);
return

function write_matlab_stream_file(fn,X,jout, tch,nonlin,names)
    ndim=length(size(X));
    if ndim==2,
       [N2,nseg]=size(X);
       dim=1;
       [nc,nseg2]=size(tch);
       n2=1;
    else,
       [N2,dim,nseg]=size(X);
       [nc,nseg2,n2]=size(tch);
    end;
    if nseg2 ~= nseg,
      error('nseg2 ~= nseg');
    end;
    nsegj=length(jout);
    if nsegj ~= nseg,
      error('nsegj ~= nseg');
    end;
    X=reshape(X,N2,dim,nseg);
    tch=reshape(tch,nc,nseg,n2);
    jout=reshape(jout,nseg,1);

    X=permute(X,[2,3,1]); % should be (dim,nseg,n2)
    tch=permute(tch,[1,2,3]); % should be (nc,nseg,N2)
    x_shape=[dim,nseg,N2];
    fprintf('saving to %s\n',fn);
    save(fn,'X','tch','jout','nonlin','names','x_shape');
return


